#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['hoplite2']

package_data = \
{'': ['*']}

install_requires = \
['Pillow >= 7.2.0', 'tensorflow >= 2.2.0']

setup(name='hoplite2',
      version='0.2.0',
      description='A Sparsity Analysis Tool and Library',
      author='Pranoy Dutta',
      author_email='pranoydll@gmail.com',
      url='https://github.com/prydt/hoplite2',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
